<div alt='{{nothing}}' class='multilinetest1' id='{{myId}}'></div>
These { { braces

should } not be interpreted as a multiline string
These { { braces&#x000A;&#x000A;should } not be interpreted as a multiline string
<style type='text/css'>
  /*<![CDATA[*/
    .test {
      display: inline;
    }
  /*]]>*/
</style>
<script type='text/javascript'>
  //<![CDATA[
    These {
    Braces should {
      also
    } be { ignored
  //]]>
</script>
<div alt='' class='{{myClass}} multilinetest2' id='{{myId}}'></div>
<!-- The following is from hjonathan, issue #67 -->
<head>
  <div class='blah'>
    <script type='text/javascript'>
      //<![CDATA[
        $(document).ready(function(){
          $("#form{{form.initial.id}}").submit(form_submit);
          //Double nesting
          $(function() {
              blahblahblah
          });

          // Javascript comment
        });
      //]]>
    </script>
  </div>
  <script type='text/javascript'>
    //<![CDATA[
      $(document).ready(function(){
        $("#form{{form.initial.id}}").submit(form_submit);
        // Javascript comment
      });
    //]]>
  </script>
  <style type='text/css'>
    /*<![CDATA[*/
      .someClass {
        width: 100px;
      }
    /*]]>*/
  </style>
  <![CDATA[
      if (a < b && a < 0)
      {
        return 1;
      }
  ]]>
</head>
<script type='text/javascript'>
  //<![CDATA[
    (
        {
            a
        }
    );
  //]]>
</script>
<script type='text/javascript'>
  //<![CDATA[

        {
            a
        }
    )
  //]]>
</script>
