<div class='true article entry hover:no-underline' id='Article_123'>
  Now this is interesting
</div>
<div class='true article entry hover:no-underline' id='Article_123'>
  Now this is really interesting
</div>
