<div class="highlight"><pre><span></span><span class="nv">print</span><span class="w"> </span><span class="s2">&quot;hi&quot;</span>

<span class="k">if</span><span class="w"> </span><span class="nv">x</span>:
<span class="w">    </span><span class="nv">print</span><span class="w"> </span><span class="s2">&quot;y&quot;</span>:
<span class="k">else</span>:
<span class="w">    </span><span class="nv">print</span><span class="w"> </span><span class="s2">&quot;z&quot;</span>:
</pre></div>

