{% extends "djangoBase.hamlpy" %}

{% block title %}
  {{ section.title }}

{% endblock %}
{% block content %}
  <h2>{{ section.subtitle }}</h2>

  {% for story in story_list %}
    <h3>
      <a href='{{ story.get_absolute_url }}'>
        {{ story.headline|upper }}
      </a>
    </h3>
    <p>{{ story.tease|truncatewords:"100" }}</p>
  {% endfor %}
{% endblock %}
