<pre>


	<div class='something'>


		Some text

		Some other text


		Some more text


	</div>
</pre>
<input type='text' />


<div class='something'>{{ var }}</div>


{% if something %}

	Test 1

	<div>


		Test 2


	</div>
{% endif %}


Some text
Some plain text

Some more text


One more text

<style type='text/css'>
  /*<![CDATA[*/

    Some text
    Some plain text

    Some more text


    One more text
  /*]]>*/
</style>
<style type='text/stylus'>
  /*<![CDATA[*/

    Some text
    Some plain text

    Some more text


    One more text
  /*]]>*/
</style>
<![CDATA[

    Some text
    Some plain text

    Some more text


    One more text
]]>
<script type='text/coffeescript'>
  #<![CDATA[

    Some text
    Some plain text

    Some more text


    One more text
  #]]>
</script>
<script type='text/javascript'>
  //<![CDATA[

    Some text
    Some plain text

    Some more text


    One more text
  //]]>
</script>

<div>Test

	asd</div>

<p>

	<li>Item one</li><li>Item two</li><li>Item three</li>

</p>
<p><li>
		Item one

	</li></p>
<p><li>Item one</li><li>Item two</li>

	<li>Item three</li>

</p>
<input /><input /><input />
