&lt;&quot;&amp;&#39;&gt;
<script type='text/javascript'>
  //<![CDATA[
    $(document).ready(function(){
      $("#form{{form.initial.id}}").submit(form_submit);
      // Javascript comment
    });
  //]]>
</script>
<style type='text/css'>
  /*<![CDATA[*/
    .someClass {
      width: 100px;
    }
  /*]]>*/
</style>
<style type='text/stylus'>
  /*<![CDATA[*/
    .someClass
      width: 100px
  /*]]>*/
</style>
<style type='text/less'>
  /*<![CDATA[*/
    .someClass {
      width: 100px;
    }
  /*]]>*/
</style>
<style type='text/sass'>
  /*<![CDATA[*/
    .someClass {
      width: 100px;
    }
  /*]]>*/
</style>
<![CDATA[
    if (a < b && a < 0)
    {
      return 1;
    }
]]>
1
2
3
4
5

